@@ -1,10 +1,11 @@
 import socket
 import sys
-import urllib.parse
 from base64 import b64encode
+from dataclasses import dataclass
 from datetime import datetime
 from ipaddress import IPv4Address, IPv6Address, ip_address
-from typing import Any, Iterable, List, Optional, Tuple, Union
+from typing import Any, Iterable, List, Optional, Union
+from urllib.parse import urlsplit
 
 import click
 from cryptography.hazmat.primitives import hashes
@@ -14,7 +15,7 @@ from cryptography.x509 import Certificate, GeneralName, Name, PolicyInformation
 from cryptography.x509.certificate_transparency import SignedCertificateTimestamp
 from OpenSSL import SSL, crypto
 
-__version__ = "2022.8.8"
+__version__ = "2022.10.24"
 
 BAD_BUYPASS_CERTS = [
     "8acd454c36e2f873c90ae6c00df75928daa414a43be745e866e8172344178824",
@@ -39,6 +40,7 @@ KNOWN_LOGS = {
     "RJRlLrDuzq/EQAfYqP4owNrmgr7YyzG1P9MzlrW2gag=": "Cloudflare 'Nimbus2021' Log",
     "QcjKsd8iRkoQxqE6CUKHXk4xixsD6+tLx2jwkGKWBvY=": "Cloudflare 'Nimbus2022' Log",
     "ejKMVNi3LbYg6jjgUh7phBZwMhOFTTvSK8E6V6NS61I=": "Cloudflare 'Nimbus2023' Log",
+    "2ra/az+1tiKfm8K7XGvocJFxbLtRhIU0vaQ9MEjX+6s=": "Cloudflare 'Nimbus2024' Log",
     "VhQGmi/XwuzT9eG9RLI+x0Z2ubyZEVzA75SYVdaJ0N0=": "DigiCert Log Server",
     "h3W/51l8+IxDmV+9827/Vo1HVjb/SrVgwbTq/16ggw8=": "DigiCert Log Server 2",
     "/kRhCLHQGreKYsz+q2qysrq/86va2ApNizDfLQAIgww=": "DigiCert Nessie2019 Log",
@@ -46,17 +48,22 @@ KNOWN_LOGS = {
     "7sCV7o1yZA+S48O5G8cSo2lqCXtLahoUOOZHssvtxfk=": "DigiCert Nessie2021 Log",
     "UaOw9f0BeZxWbbg3eI8MpHrMGyfL956IQpoN/tSLBeU=": "DigiCert Nessie2022 Log",
     "s3N3B+GEUPhjhtYFqdwRCUp5LbFnDAuH3PADDnk2pZo=": "DigiCert Nessie2023 Log",
+    "c9meiRtMlnigIH1HneayxhzQUV5xGSqMa4AQesF3crU=": "DigiCert Nessie2024 Log",
+    "5tIxY0B3jMEQQQbXcbnOwdJA9paEhvu6hzId/R43jlA=": "DigiCert Nessie2025 Log",
     "4mlLribo6UAJ6IYbtjuD1D7n/nSI+6SPKJMBnd3x2/4=": "DigiCert Yeti2019 Log",
     "8JWkWfIA0YJAEC0vk4iOrUv+HUfjmeHQNKawqKqOsnM=": "DigiCert Yeti2020 Log",
     "XNxDkv7mq0VEsV6a1FbmEDf71fpH3KFzlLJe5vbHDso=": "DigiCert Yeti2021 Log",
     "IkVFB1lVJFaWP6Ev8fdthuAjJmOtwEt/XcaDXG7iDwI=": "DigiCert Yeti2022 Log",
     "BZwB0yDgB4QTlYBJjRF8kDJmr69yULWvO0akPhGEDUo=": "DigiCert Yeti2022-2 Log",
     "Nc8ZG7+xbFe/D61MbULLu7YnICZR6j/hKu+oA8M71kw=": "DigiCert Yeti2023 Log",
+    "SLDja9qmRzQP5WoC+p0w6xxSActW3SyB2bu/qznYhHM=": "DigiCert Yeti2024 Log",
+    "fVkeEuF4KnscYWd8Xv340IdcFKBOlZ65Ay/ZDowuebg=": "DigiCert Yeti2025 Log",
     "Y/Lbzeg7zCzPC3KEJ1drM6SNYXePvXWmOLHHaFRL2I0=": "Google 'Argon2019' log",
     "sh4FzIuizYogTodm+Su5iiUgZ2va+nDnsklTLe+LkF4=": "Google 'Argon2020' log",
     "9lyUL9F3MCIUVBgIMJRWjuNNExkzv98MLyALzE7xZOM=": "Google 'Argon2021' log",
     "KXm+8J45OSHwVnOfY6V35b5XfZxgCvj5TV0mXCVdx4Q=": "Google 'Argon2022' log",
     "6D7Q2j71BjUy51covIlryQPTy9ERa+zraeF3fW0GvW4=": "Google 'Argon2023' log",
+    "7s3QZNXbGs7FXLedtM0TojKHRny87N7DUUhZRnEftZs=": "Google 'Argon2024' log",
     "aPaY+B9kgr46jO65KB1M/HFRXWeT1ETRCmesu09P+8Q=": "Google 'Aviator' log",
     "KTxRllTIOWW6qlD8WAfUt2+/WHopctykwwz05UVH9Hg=": "Google 'Icarus' log",
     "pLkJkLQYWBSHuxOizGdwCjw1mAT5G9+443fNDsgN3BA=": "Google 'Pilot' log",
@@ -67,10 +74,13 @@ KNOWN_LOGS = {
     "fT7y+I//iFVoJMLAyp5SiXkrxQ54CX8uapdomX4i8Nc=": "Google 'Xenon2021' log",
     "RqVV63X6kSAwtaKJafTzfREsQXS+/Um4havy/HD+bUc=": "Google 'Xenon2022' log",
     "rfe++nz/EMiLnT2cHj4YarRnKV3PsQwkyoWGNOvcgoo=": "Google 'Xenon2023' log",
+    "dv+IPwq2+5VRwmHM9Ye6NLSkzbsp3GhCCp/mZ0xaOnQ=": "Google 'Xenon2024' log",
     "dGG0oJz7PUHXUVlXWy52SaRFqNJ3CbDMVkpkgrfrQaM=": "Izenpe log",
     "lCC8Ho7VjWyIcx+CiyIsDdHaTV5sT5Q9YdtOL1hNosI=": "Let's Encrypt 'Oak2021' log",
     "36Veq2iCTx9sre64X04+WurNohKkal6OOxLAIERcKnM=": "Let's Encrypt 'Oak2022' log",
     "tz77JN+cTbp18jnFulj0bF38Qs96nzXEnh0JgSXttJk=": "Let's Encrypt 'Oak2023' log",
+    "O1N3dT4tuYBOizBbBv5AO2fYT8P0x70ADS1yb+H61Bc=": "Let's Encrypt 'Oak2024H1' log",
+    "PxdLT9ciR1iUHWUchL4NEu2QN38fhWrrwb8ohez4ZG4=": "Let's Encrypt 'Oak2024H2' log",
     "b1N2rDHwMRnYmQCkURX/dxUcEdkCwQApBo2yCJo32RM=": "Sectigo 'Mammoth' CT log",
     "VYHUwhaQNgFK6gubVzxT8MDkOHhwJQgXL6OqHQcT0ww=": "Sectigo 'Sabre' CT log",
     "NLtq1sPfnAPuqKSZ/3iRSGydXlysktAfe/0bzhnbSO8=": "StartCom log",
@@ -93,6 +103,21 @@ KNOWN_CERT_TYPES = {
 }
 
 
+@dataclass
+class Host:
+    host: Union[str, IPv4Address, IPv6Address]
+    port: int
+
+    def __str__(self) -> str:
+        if isinstance(self.host, IPv6Address):
+            return f"[{self.host}]:{self.port}"
+        return f"{self.host}:{self.port}"
+
+    @property
+    def is_ip(self) -> bool:
+        return isinstance(self.host, (IPv4Address, IPv6Address))
+
+
 @click.command(context_settings={"help_option_names": ["-h", "--help"]})
 @click.version_option(version=__version__)
 @click.argument("host")
@@ -112,52 +137,33 @@ def main(
     print_pem: bool,
     first_only: bool,
     openssl_format: bool,
-):
+) -> None:
     """Peeks at certificates exposed by other hosts."""
     if servername and no_servername:
         raise click.BadArgumentUsage(
             "--servername and --no-servername are mutually exclusive."
         )
 
-    # TODO: add support for IPv6
-    host_split = host.split(":")
-    if len(host_split) == 1:
-        host_and_port = host_split[0], 443
-    elif len(host_split) == 2:
-        try:
-            port = int(host_split[1])
-        except ValueError:
-            raise click.BadParameter("Port must be integer: {}".format(host_split[1]))
-        host_and_port = host_split[0], port
-    else:
-        raise click.BadParameter("Invalid host specified")
+    parsed_host = parse_host_input(host)
 
     if proxy:
         click.secho(f"Connecting via '{proxy}'", err=True)
-        s = get_socket_via_proxy(proxy, host_and_port)
+        s = get_socket_via_proxy(proxy, parsed_host)
     else:
-        click.secho(f"Connecting directly to host '{host_and_port[0]}'", err=True)
-        s = get_direct_socket(host_and_port)
+        click.secho(f"Connecting directly to host '{parsed_host}'", err=True)
+        s = get_direct_socket(parsed_host)
 
     ctx = SSL.Context(SSL.SSLv23_METHOD)
     conn = SSL.Connection(ctx, s)
 
-    name_to_use: str = servername if servername else host_and_port[0]
-    destination: Union[str, IPv4Address, IPv6Address]
-    try:
-        destination = ip_address(name_to_use)
-    except ValueError:
-        # hostname, not an ip
-        destination = name_to_use
-
     if not no_servername:
         if servername:
             conn.set_tlsext_host_name(servername.encode())
         else:
             # IP addresses are not permitted in servername
-            # so only add if we are connecting do a DNS name.
-            if isinstance(destination, str):
-                conn.set_tlsext_host_name(destination.encode())
+            # so only add if we are connecting to a DNS name.
+            if not parsed_host.is_ip:
+                conn.set_tlsext_host_name(str(parsed_host.host).encode())
 
     conn.set_connect_state()
     try:
@@ -168,7 +174,7 @@ def main(
         # If the host requires a client certificate
         # the handshake will fail, but we will still
         # get our certificate.
-        ssl_error = error
+        ssl_error: Optional[SSL.Error] = error
     else:
         ssl_error = None
 
@@ -189,7 +195,7 @@ def main(
             click.echo(crypto.dump_certificate(crypto.FILETYPE_TEXT, cert).decode())
         else:
             last_issuer = print_cert_info(
-                cert.to_cryptography(), destination, last_issuer
+                cert.to_cryptography(), parsed_host.host, last_issuer
             )
         if print_pem:
             pem_cert = crypto.dump_certificate(crypto.FILETYPE_PEM, cert)
@@ -199,61 +205,89 @@ def main(
             break
 
 
-def get_socket_via_proxy(proxy: str, host: Tuple[str, int]):
+def parse_host_input(input: str) -> Host:
+    # A bare IPv6 address can be confused
+    # with a host:port combo, so let's try
+    # to parse it as that first.
+    try:
+        return Host(ip_address(input), 443)
+    except ValueError:
+        pass
 
-    proxy_addr = urllib.parse.urlparse(proxy)
+    parsed_host = urlsplit(input)
+    if not parsed_host.netloc:
+        parsed_host = urlsplit(f"//{input}")
+
+    if not parsed_host.hostname:
+        raise click.BadParameter("Invalid host specified")
+
+    try:
+        port = parsed_host.port
+    except ValueError:
+        raise click.BadParameter("Invalid port specified")
+
+    if port is None:
+        # default to 443, or whatever is default for the
+        # specified schema (if we know it)
+        port = 443
+        if parsed_host.scheme:
+            try:
+                port = socket.getservbyname(parsed_host.scheme)
+            except OSError:
+                # unknown scheme
+                pass
+
+    try:
+        return Host(ip_address(parsed_host.hostname), port)
+    except ValueError:
+        return Host(parsed_host.hostname, port)
+
+
+def get_socket_via_proxy(proxy: str, host: Host) -> socket.socket:
+
+    proxy_addr = urlsplit(proxy)
     if proxy_addr.scheme != "http":
         raise click.BadParameter("Only http proxies are supported")
 
-    proxy_split = proxy_addr.netloc.split(":")
-    if len(proxy_split) == 1:
-        proxy_host = proxy_split[0]
-        proxy_port = 8080
-    elif len(proxy_split) == 2:
-        try:
-            port = int(proxy_split[1])
-        except ValueError:
-            raise click.BadParameter(
-                "Proxy port must be integer: {}".format(proxy_split[1])
-            )
+    proxy_host = proxy_addr.hostname
+    try:
+        proxy_port = proxy_addr.port or 8080
+    except ValueError:
+        raise click.BadParameter("Invalid proxy port specified")
 
-        proxy_host = proxy_split[0]
-        proxy_port = port
-    else:
+    if proxy_host is None:
         raise click.BadParameter("Invalid proxy specified")
 
-    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-
     try:
-        s.connect((proxy_host, proxy_port))
+        s = socket.create_connection((proxy_host, proxy_port))
     except socket.error as error:
-        click.secho(
-            "Unable to connect to {}: {}".format(proxy, error), fg="red", err=True
-        )
+        click.secho(f"Unable to connect to proxy {proxy}: {error}", fg="red", err=True)
         sys.exit(2)
 
-    s.send("CONNECT {0}:{1} HTTP/1.1\r\nHost: {0}\r\n\r\n".format(*host).encode())
-    proxy_response = s.recv(1024).decode()
-    status_code = proxy_response.split("\r\n")[0].split(" ")[1]
+    s.send(f"CONNECT {host} HTTP/1.1\r\nHost: {host}\r\n\r\n".encode())
+    try:
+        proxy_response = s.recv(1024).decode()
+        status_code = proxy_response.split("\r\n")[0].split(" ")[1]
+    except (UnicodeDecodeError, IndexError):
+        click.secho(f"Recieved invalid response from proxy {proxy}", fg="red", err=True)
+        sys.exit(5)
+
     if status_code != "200":
-        click.secho("Computer says no:\n{}".format(proxy_response), fg="red", err=True)
+        click.secho(f"Computer says no:\n{proxy_response}", fg="red", err=True)
         sys.exit(3)
     return s
 
 
-def get_direct_socket(host: Tuple[str, int]):
-    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+def get_direct_socket(host: Host) -> socket.socket:
     try:
-        s.connect(host)
+        s = socket.create_connection((str(host.host), host.port))
     except socket.error as error:
-        click.secho(
-            "Unable to connect to {}:{} {}".format(*host, error), fg="red", err=True
-        )
+        click.secho(f"Unable to connect to {host}: {error}", fg="red", err=True)
         sys.exit(4)
     return s
 
 
-def print_field(header: str, values: Iterable[Union[str, int, None]]):
+def print_field(header: str, values: Iterable[Union[str, int, None]]) -> None:
     if values and any(values):
         click.secho("[{}]".format(header))
         for value in values:
@@ -297,7 +331,7 @@ def get_not_after_status(not_after: datetime) -> str:
     return "{} ({})".format(not_after, text)
 
 
-def get_hash_algorithm_name(cert: Certificate):
+def get_hash_algorithm_name(cert: Certificate) -> Optional[str]:
     return cert.signature_hash_algorithm.name if cert.signature_hash_algorithm else None
 
 
@@ -323,7 +357,7 @@ def print_cert_info(
     cert: Certificate,
     destination: Union[str, IPv4Address, IPv6Address],
     last_issuer: Optional[Name],
-):
+) -> Name:
     sans: List[str] = []
     scts: List[SignedCertificateTimestamp] = []
     policies: List[PolicyInformation] = []
